/*
   CScanner - A Chinese Lexical Scanner
   Copyright (C) 2000-2001 Chih-Hao Tsai

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.
 */

/*
   Module: cscanner.h
   Version: 0.26
   Date: April 7, 2001
   Author: Chih-Hao Tsai
   Email: hao520@yahoo.com
   CScanner website: http://technology.chtsai.org/cscanner/
   Author's homepage: http://chtsai.org/
 */

#ifndef __CSCANNER_H__		/* include guard */
#define __CSCANNER_H__

#ifdef __cplusplus
extern "C"
{				/* facility for C++ friendly */
#endif

#include "skiplists.h"

#define TOKEN_LENGTH_MAX (64 * sizeof(char))	/* Maximum length of a token */


/* Definition of a lexicon */
  typedef struct lexicon Lexicon;
  struct lexicon
  {
    int size;			/* total number of words */
    Skiplists Word;		/* an ordered list of words (skip lists) */
    unsigned char *charfreq;	/* an array of character frequencies */
  };

/* Token types */
  typedef enum
  {
    UNRECOGNIZED, ASCII_WORD, ASCII_SYMBOL, ASCII_8BIT, BIG5_WORD,
      BIG5_SYMBOL, BIG5_UNDEFINED
  }
  Tokentype;

/* Definition of a CScanner object */
  typedef struct cscanner CScanner;
  struct cscanner
  {
    const char *text;		/* text to be scanned */
    unsigned int length;	/* how far the scanner will go */
    int token_length_max;	/* Maximum length of a token */
    int chinese_word_length_max;	/* Maximum length of a Chinese word */
    const Lexicon *lexicon;     /* pointer to the lexicon */
    char token[TOKEN_LENGTH_MAX + sizeof (char)];	/* current token */
    Tokentype type;		/* current token type */
    unsigned int token_position;	/* position of current token */
    unsigned int position;	/* starting position of next scan */
  };

/* Main CScanner functions */
  CScanner *c_scanner_new (Lexicon *);
  int c_scanner_input_text (CScanner *, const char *, int);
  char *c_scanner_get_token (CScanner *, int);
  void c_scanner_free (CScanner *);

/* BIG5 handling functions */
  int is_big5_leading_char (unsigned char);
  int is_big5_trailing_char (unsigned char);
  int is_big5_chinese_char (unsigned char, unsigned char);
  int is_big5_symbol (unsigned char, unsigned char);
  int is_big5_undefined_char (unsigned char, unsigned char);

/* Lexicon functions */
  Lexicon *c_scanner_load_lexicon (const char *);
  void *c_scanner_search_lexicon (char *, const Lexicon *);
  void c_scanner_free_lexicon (Lexicon *);

#ifdef __cplusplus
}
#endif				/* extern "C" */

#endif				/* __CSCANNER_H__ */
